@@ -39,9 +39,6 @@ transform
     Geometric and other transforms, e.g., rotation or the Radon transform.
 util
     Generic utilities.
-viewer
-    A simple graphical user interface for visualizing results and exploring
-    parameters.
 
 Utility Functions
 -----------------
@@ -68,43 +65,17 @@ dtype_limits
 
 """
 
-__version__ = '0.19.3'
+__version__ = '0.23.2'
 
-submodules = [
-    'color',
-    'data',
-    'draw',
-    'exposure',
-    'feature',
-    'filters',
-    'future',
-    'graph',
-    'io',
-    'measure',
-    'metrics',
-    'morphology',
-    'registration',
-    'restoration',
-    'segmentation',
-    'transform',
-    'util',
-    'viewer'
-]
+import lazy_loader as lazy
 
-from ._shared.version_requirements import ensure_python_version
-ensure_python_version((3, 7))
-
-from ._shared import lazy
-__getattr__, __lazy_dir__, _ = lazy.attach(
-    __name__,
-    submodules,
-    submod_attrs={'data': ['data_dir']}
-)
+__getattr__, __lazy_dir__, _ = lazy.attach_stub(__name__, __file__)
 
 
 def __dir__():
     return __lazy_dir__() + ['__version__']
 
+
 # Logic for checking for improper install and importing while in the source
 # tree when package has not been installed inplace.
 # Code adapted from scikit-learn's __check_build module.
@@ -116,21 +87,22 @@ directory and you need to try from another location."""
 _STANDARD_MSG = """
 Your install of scikit-image appears to be broken.
 Try re-installing the package following the instructions at:
-https://scikit-image.org/docs/stable/install.html """
+https://scikit-image.org/docs/stable/user_guide/install.html"""
 
 
 def _raise_build_error(e):
     # Raise a comprehensible error
     import os.path as osp
+
     local_dir = osp.split(__file__)[0]
     msg = _STANDARD_MSG
     if local_dir == "skimage":
         # Picking up the local install: this will work only if the
         # install is an 'inplace build'
         msg = _INPLACE_MSG
-    raise ImportError("""%s
-It seems that scikit-image has not been built correctly.
-%s""" % (e, msg))
+    raise ImportError(
+        f"{e}\nIt seems that scikit-image has not been built correctly.\n{msg}"
+    )
 
 
 try:
@@ -143,27 +115,35 @@ except NameError:
 
 if __SKIMAGE_SETUP__:
     import sys
+
     sys.stderr.write('Partial import of skimage during the build process.\n')
     # We are not importing the rest of the scikit during the build
     # process, as it may not be compiled yet
 else:
     try:
         from ._shared import geometry
+
         del geometry
     except ImportError as e:
         _raise_build_error(e)
 
-    # All skimage root imports go here
-    from .util.dtype import (img_as_float32,
-                             img_as_float64,
-                             img_as_float,
-                             img_as_int,
-                             img_as_uint,
-                             img_as_ubyte,
-                             img_as_bool,
-                             dtype_limits)
+    # Legacy imports into the root namespace; not advertised in __all__
+    from .util.dtype import (
+        dtype_limits,
+        img_as_float32,
+        img_as_float64,
+        img_as_float,
+        img_as_int,
+        img_as_uint,
+        img_as_ubyte,
+        img_as_bool,
+    )
+
     from .util.lookfor import lookfor
 
+    from .data import data_dir
+
+
 if 'dev' in __version__:
     # Append last commit date and hash to dev version information, if available
 
@@ -192,11 +172,11 @@ if 'dev' in __version__:
             )
 
             __version__ = '+'.join(
-                [tag for tag in __version__.split('+')
-                 if not tag.startswith('git')]
+                [tag for tag in __version__.split('+') if not tag.startswith('git')]
             )
             __version__ += f'+git{git_date}.{git_hash}'
 
 from skimage._shared.tester import PytestTester  # noqa
+
 test = PytestTester(__name__)
 del PytestTester
