@@ -12,7 +12,12 @@ from scipy.spatial.distance import pdist
 from . import _moments
 from ._find_contours import find_contours
 from ._marching_cubes_lewiner import marching_cubes
-from ._regionprops_utils import euler_number, perimeter, perimeter_crofton
+from ._regionprops_utils import (
+    euler_number,
+    perimeter,
+    perimeter_crofton,
+    _normalize_spacing,
+)
 
 __all__ = ['regionprops', 'euler_number', 'perimeter', 'perimeter_crofton']
 
@@ -63,6 +68,7 @@ PROPS = {
     'mean_intensity': 'intensity_mean',
     'MinIntensity': 'intensity_min',
     'min_intensity': 'intensity_min',
+    'std_intensity': 'intensity_std',
     'MinorAxisLength': 'axis_minor_length',
     'minor_axis_length': 'axis_minor_length',
     'Moments': 'moments',
@@ -89,16 +95,11 @@ PROPS = {
     'weighted_moments_normalized': 'moments_weighted_normalized',
 }
 
-OBJECT_COLUMNS = {
-    'image', 'coords', 'image_convex', 'slice',
-    'image_filled', 'image_intensity'
-}
-
 COL_DTYPES = {
-    'area': int,
-    'area_bbox': int,
-    'area_convex': int,
-    'area_filled': int,
+    'area': float,
+    'area_bbox': float,
+    'area_convex': float,
+    'area_filled': float,
     'axis_major_length': float,
     'axis_minor_length': float,
     'bbox': int,
@@ -107,6 +108,7 @@ COL_DTYPES = {
     'centroid_weighted': float,
     'centroid_weighted_local': float,
     'coords': object,
+    'coords_scaled': object,
     'eccentricity': float,
     'equivalent_diameter_area': float,
     'euler_number': int,
@@ -121,6 +123,7 @@ COL_DTYPES = {
     'intensity_max': float,
     'intensity_mean': float,
     'intensity_min': float,
+    'intensity_std': float,
     'label': int,
     'moments': float,
     'moments_central': float,
@@ -130,6 +133,7 @@ COL_DTYPES = {
     'moments_weighted_central': float,
     'moments_weighted_hu': float,
     'moments_weighted_normalized': float,
+    'num_pixels': int,
     'orientation': float,
     'perimeter': float,
     'perimeter_crofton': float,
@@ -137,8 +141,24 @@ COL_DTYPES = {
     'solidity': float,
 }
 
+OBJECT_COLUMNS = [col for col, dtype in COL_DTYPES.items() if dtype == object]
+
 PROP_VALS = set(PROPS.values())
 
+_require_intensity_image = (
+    'image_intensity',
+    'intensity_max',
+    'intensity_mean',
+    'intensity_min',
+    'intensity_std',
+    'moments_weighted',
+    'moments_weighted_central',
+    'centroid_weighted',
+    'centroid_weighted_local',
+    'moments_weighted_hu',
+    'moments_weighted_normalized',
+)
+
 
 def _infer_number_of_required_args(func):
     """Infer the number of required arguments for a function
@@ -182,22 +202,27 @@ def _infer_regionprop_dtype(func, *, intensity, ndim):
     dtype : NumPy data type
         The data type of the returned property.
     """
-    labels = [1, 2]
-    sample = np.zeros((3,) * ndim, dtype=np.intp)
-    sample[(0,) * ndim] = labels[0]
-    sample[(slice(1, None),) * ndim] = labels[1]
-    propmasks = [(sample == n) for n in labels]
+    mask_1 = np.ones((1,) * ndim, dtype=bool)
+    mask_1 = np.pad(mask_1, (0, 1), constant_values=False)
+    mask_2 = np.ones((2,) * ndim, dtype=bool)
+    mask_2 = np.pad(mask_2, (1, 0), constant_values=False)
+    propmasks = [mask_1, mask_2]
 
     rng = np.random.default_rng()
 
     if intensity and _infer_number_of_required_args(func) == 2:
+
         def _func(mask):
-            return func(mask, rng.random(sample.shape))
+            return func(mask, rng.random(mask.shape))
+
     else:
         _func = func
     props1, props2 = map(_func, propmasks)
-    if (np.isscalar(props1) and np.isscalar(props2)
-            or np.array(props1).shape == np.array(props2).shape):
+    if (
+        np.isscalar(props1)
+        and np.isscalar(props2)
+        or np.array(props1).shape == np.array(props2).shape
+    ):
         dtype = np.array(props1).dtype.type
     else:
         dtype = np.object_
@@ -210,7 +235,10 @@ def _cached(f):
         cache = obj._cache
         prop = f.__name__
 
-        if not ((prop in cache) and obj._cache_active):
+        if not obj._cache_active:
+            return f(obj)
+
+        if prop not in cache:
             cache[prop] = f(obj)
 
         return cache[prop]
@@ -222,18 +250,20 @@ def only2d(method):
     @wraps(method)
     def func2d(self, *args, **kwargs):
         if self._ndim > 2:
-            raise NotImplementedError('Property %s is not implemented for '
-                                      '3D images' % method.__name__)
+            raise NotImplementedError(
+                f"Property {method.__name__} is not implemented for 3D images"
+            )
         return method(self, *args, **kwargs)
+
     return func2d
 
 
 def _inertia_eigvals_to_axes_lengths_3D(inertia_tensor_eigvals):
     """Compute ellipsoid axis lengths from inertia tensor eigenvalues.
 
-    Paramters
+    Parameters
     ---------
-    inertia_tensor_eigvals : seqeunce of float
+    inertia_tensor_eigvals : sequence of float
         A sequence of 3 floating point eigenvalues, sorted in descending order.
 
     Returns
@@ -265,8 +295,7 @@ def _inertia_eigvals_to_axes_lengths_3D(inertia_tensor_eigvals):
     """
     axis_lengths = []
     for ax in range(2, -1, -1):
-        w = sum(v * -1 if i == ax else v
-                for i, v in enumerate(inertia_tensor_eigvals))
+        w = sum(v * -1 if i == ax else v for i, v in enumerate(inertia_tensor_eigvals))
         axis_lengths.append(sqrt(10 * w))
     return axis_lengths
 
@@ -276,22 +305,36 @@ class RegionProperties:
     on the available region properties.
     """
 
-    def __init__(self, slice, label, label_image, intensity_image,
-                 cache_active, *, extra_properties=None):
-
+    def __init__(
+        self,
+        slice,
+        label,
+        label_image,
+        intensity_image,
+        cache_active,
+        *,
+        extra_properties=None,
+        spacing=None,
+        offset=None,
+    ):
         if intensity_image is not None:
             ndim = label_image.ndim
             if not (
-                    intensity_image.shape[:ndim] == label_image.shape
-                    and intensity_image.ndim in [ndim, ndim + 1]
-                    ):
-                raise ValueError('Label and intensity image shapes must match,'
-                                 ' except for channel (last) axis.')
+                intensity_image.shape[:ndim] == label_image.shape
+                and intensity_image.ndim in [ndim, ndim + 1]
+            ):
+                raise ValueError(
+                    'Label and intensity image shapes must match,'
+                    ' except for channel (last) axis.'
+                )
             multichannel = label_image.shape < intensity_image.shape
         else:
             multichannel = False
 
         self.label = label
+        if offset is None:
+            offset = np.zeros((label_image.ndim,), dtype=int)
+        self._offset = np.array(offset)
 
         self._slice = slice
         self.slice = slice
@@ -303,6 +346,10 @@ class RegionProperties:
         self._ndim = label_image.ndim
         self._multichannel = multichannel
         self._spatial_axes = tuple(range(self._ndim))
+        if spacing is None:
+            spacing = np.full(self._ndim, 1.0)
+        self._spacing = _normalize_spacing(spacing, self._ndim)
+        self._pixel_area = np.prod(self._spacing)
 
         self._extra_properties = {}
         if extra_properties is not None:
@@ -315,11 +362,14 @@ class RegionProperties:
                         f"renaming it."
                     )
                     warn(msg)
-            self._extra_properties = {
-                func.__name__: func for func in extra_properties
-            }
+            self._extra_properties = {func.__name__: func for func in extra_properties}
 
     def __getattr__(self, attr):
+        if self._intensity_image is None and attr in _require_intensity_image:
+            raise AttributeError(
+                f"Attribute '{attr}' unavailable when `intensity_image` "
+                f"has not been specified."
+            )
         if attr in self._extra_properties:
             func = self._extra_properties[attr]
             n_args = _infer_number_of_required_args(func)
@@ -327,10 +377,10 @@ class RegionProperties:
             if n_args == 2:
                 if self._intensity_image is not None:
                     if self._multichannel:
-                        multichannel_list = [func(self.image,
-                                                  self.image_intensity[..., i])
-                                             for i in range(
-                            self.image_intensity.shape[-1])]
+                        multichannel_list = [
+                            func(self.image, self.image_intensity[..., i])
+                            for i in range(self.image_intensity.shape[-1])
+                        ]
                         return np.stack(multichannel_list, axis=-1)
                     else:
                         return func(self.image, self.image_intensity)
@@ -346,12 +396,18 @@ class RegionProperties:
                     f'be 1 or 2, but {attr} takes {n_args} arguments.'
                 )
         elif attr in PROPS and attr.lower() == attr:
+            if (
+                self._intensity_image is None
+                and PROPS[attr] in _require_intensity_image
+            ):
+                raise AttributeError(
+                    f"Attribute '{attr}' unavailable when `intensity_image` "
+                    f"has not been specified."
+                )
             # retrieve deprecated property (excluding old CamelCase ones)
             return getattr(self, PROPS[attr])
         else:
-            raise AttributeError(
-                f"'{type(self)}' object has no attribute '{attr}'"
-            )
+            raise AttributeError(f"'{type(self)}' object has no attribute '{attr}'")
 
     def __setattr__(self, name, value):
         if name in PROPS:
@@ -361,9 +417,14 @@ class RegionProperties:
 
     @property
     @_cached
-    def area(self):
+    def num_pixels(self):
         return np.sum(self.image)
 
+    @property
+    @_cached
+    def area(self):
+        return np.sum(self.image) * self._pixel_area
+
     @property
     def bbox(self):
         """
@@ -372,33 +433,42 @@ class RegionProperties:
         A tuple of the bounding box's start coordinates for each dimension,
         followed by the end coordinates for each dimension
         """
-        return tuple([self.slice[i].start for i in range(self._ndim)] +
-                     [self.slice[i].stop for i in range(self._ndim)])
+        return tuple(
+            [self.slice[i].start for i in range(self._ndim)]
+            + [self.slice[i].stop for i in range(self._ndim)]
+        )
 
     @property
     def area_bbox(self):
-        return self.image.size
+        return self.image.size * self._pixel_area
 
     @property
     def centroid(self):
-        return tuple(self.coords.mean(axis=0))
+        return tuple(self.coords_scaled.mean(axis=0))
 
     @property
     @_cached
     def area_convex(self):
-        return np.sum(self.image_convex)
+        return np.sum(self.image_convex) * self._pixel_area
 
     @property
     @_cached
     def image_convex(self):
         from ..morphology.convex_hull import convex_hull_image
+
         return convex_hull_image(self.image)
 
+    @property
+    def coords_scaled(self):
+        indices = np.argwhere(self.image)
+        object_offset = np.array([self.slice[i].start for i in range(self._ndim)])
+        return (object_offset + indices) * self._spacing + self._offset
+
     @property
     def coords(self):
-        indices = np.nonzero(self.image)
-        return np.vstack([indices[i] + self.slice[i].start
-                          for i in range(self._ndim)]).T
+        indices = np.argwhere(self.image)
+        object_offset = np.array([self.slice[i].start for i in range(self._ndim)])
+        return object_offset + indices + self._offset
 
     @property
     @only2d
@@ -415,30 +485,32 @@ class RegionProperties:
     @property
     def euler_number(self):
         if self._ndim not in [2, 3]:
-            raise NotImplementedError('Euler number is implemented for '
-                                      '2D or 3D images only')
+            raise NotImplementedError(
+                'Euler number is implemented for ' '2D or 3D images only'
+            )
         return euler_number(self.image, self._ndim)
 
     @property
     def extent(self):
-        return self.area / self.image.size
+        return self.area / self.area_bbox
 
     @property
     def feret_diameter_max(self):
-        identity_convex_hull = np.pad(self.image_convex,
-                                      2, mode='constant', constant_values=0)
+        identity_convex_hull = np.pad(
+            self.image_convex, 2, mode='constant', constant_values=0
+        )
         if self._ndim == 2:
-            coordinates = np.vstack(find_contours(identity_convex_hull, .5,
-                                                  fully_connected='high'))
+            coordinates = np.vstack(
+                find_contours(identity_convex_hull, 0.5, fully_connected='high')
+            )
         elif self._ndim == 3:
-            coordinates, _, _, _ = marching_cubes(identity_convex_hull,
-                                                  level=.5)
-        distances = pdist(coordinates, 'sqeuclidean')
+            coordinates, _, _, _ = marching_cubes(identity_convex_hull, level=0.5)
+        distances = pdist(coordinates * self._spacing, 'sqeuclidean')
         return sqrt(np.max(distances))
 
     @property
     def area_filled(self):
-        return np.sum(self.image_filled)
+        return np.sum(self.image_filled) * self._pixel_area
 
     @property
     @_cached
@@ -455,13 +527,12 @@ class RegionProperties:
     @_cached
     def inertia_tensor(self):
         mu = self.moments_central
-        return _moments.inertia_tensor(self.image, mu)
+        return _moments.inertia_tensor(self.image, mu, spacing=self._spacing)
 
     @property
     @_cached
     def inertia_tensor_eigvals(self):
-        return _moments.inertia_tensor_eigvals(self.image,
-                                               T=self.inertia_tensor)
+        return _moments.inertia_tensor_eigvals(self.image, T=self.inertia_tensor)
 
     @property
     @_cached
@@ -476,7 +547,7 @@ class RegionProperties:
         return self._intensity_image[self.slice] * image
 
     def _image_intensity_double(self):
-        return self.image_intensity.astype(np.double, copy=False)
+        return self.image_intensity.astype(np.float64, copy=False)
 
     @property
     def centroid_local(self):
@@ -487,12 +558,13 @@ class RegionProperties:
             return (0,) * axis + (1,) + (0,) * (self._ndim - 1 - axis)
 
         return np.asarray(
-            tuple(M[_get_element(axis)] / M0 for axis in range(self._ndim)))
+            tuple(M[_get_element(axis)] / M0 for axis in range(self._ndim))
+        )
 
     @property
     def intensity_max(self):
         vals = self.image_intensity[self.image]
-        return np.max(vals, axis=0).astype(np.double, copy=False)
+        return np.max(vals, axis=0).astype(np.float64, copy=False)
 
     @property
     def intensity_mean(self):
@@ -501,7 +573,12 @@ class RegionProperties:
     @property
     def intensity_min(self):
         vals = self.image_intensity[self.image]
-        return np.min(vals, axis=0).astype(np.double, copy=False)
+        return np.min(vals, axis=0).astype(np.float64, copy=False)
+
+    @property
+    def intensity_std(self):
+        vals = self.image_intensity[self.image]
+        return np.std(vals, axis=0)
 
     @property
     def axis_major_length(self):
@@ -530,25 +607,33 @@ class RegionProperties:
     @property
     @_cached
     def moments(self):
-        M = _moments.moments(self.image.astype(np.uint8), 3)
+        M = _moments.moments(self.image.astype(np.uint8), 3, spacing=self._spacing)
         return M
 
     @property
     @_cached
     def moments_central(self):
-        mu = _moments.moments_central(self.image.astype(np.uint8),
-                                      self.centroid_local, order=3)
+        mu = _moments.moments_central(
+            self.image.astype(np.uint8),
+            self.centroid_local,
+            order=3,
+            spacing=self._spacing,
+        )
         return mu
 
     @property
     @only2d
     def moments_hu(self):
+        if any(s != 1.0 for s in self._spacing):
+            raise NotImplementedError('`moments_hu` supports spacing = (1, 1) only')
         return _moments.moments_hu(self.moments_normalized)
 
     @property
     @_cached
     def moments_normalized(self):
-        return _moments.moments_normalized(self.moments_central, 3)
+        return _moments.moments_normalized(
+            self.moments_central, 3, spacing=self._spacing
+        )
 
     @property
     @only2d
@@ -556,21 +641,25 @@ class RegionProperties:
         a, b, b, c = self.inertia_tensor.flat
         if a - c == 0:
             if b < 0:
-                return -PI / 4.
+                return PI / 4.0
             else:
-                return PI / 4.
+                return -PI / 4.0
         else:
             return 0.5 * atan2(-2 * b, c - a)
 
     @property
     @only2d
     def perimeter(self):
-        return perimeter(self.image, 4)
+        if len(np.unique(self._spacing)) != 1:
+            raise NotImplementedError('`perimeter` supports isotropic spacings only')
+        return perimeter(self.image, 4) * self._spacing[0]
 
     @property
     @only2d
     def perimeter_crofton(self):
-        return perimeter_crofton(self.image, 4)
+        if len(np.unique(self._spacing)) != 1:
+            raise NotImplementedError('`perimeter` supports isotropic spacings only')
+        return perimeter_crofton(self.image, 4) * self._spacing[0]
 
     @property
     def solidity(self):
@@ -579,8 +668,10 @@ class RegionProperties:
     @property
     def centroid_weighted(self):
         ctr = self.centroid_weighted_local
-        return tuple(idx + slc.start
-                     for idx, slc in zip(ctr, self.slice))
+        return tuple(
+            idx + slc.start * spc
+            for idx, slc, spc in zip(ctr, self.slice, self._spacing)
+        )
 
     @property
     def centroid_weighted_local(self):
@@ -591,7 +682,8 @@ class RegionProperties:
             return (0,) * axis + (1,) + (0,) * (self._ndim - 1 - axis)
 
         return np.asarray(
-            tuple(M[_get_element(axis)] / M0 for axis in range(self._ndim)))
+            tuple(M[_get_element(axis)] / M0 for axis in range(self._ndim))
+        )
 
     @property
     @_cached
@@ -599,12 +691,14 @@ class RegionProperties:
         image = self._image_intensity_double()
         if self._multichannel:
             moments = np.stack(
-                    [_moments.moments(image[..., i], order=3)
-                        for i in range(image.shape[-1])],
-                    axis=-1,
-                    )
+                [
+                    _moments.moments(image[..., i], order=3, spacing=self._spacing)
+                    for i in range(image.shape[-1])
+                ],
+                axis=-1,
+            )
         else:
-            moments = _moments.moments(image, order=3)
+            moments = _moments.moments(image, order=3, spacing=self._spacing)
         return moments
 
     @property
@@ -615,18 +709,22 @@ class RegionProperties:
         if self._multichannel:
             moments_list = [
                 _moments.moments_central(
-                    image[..., i], center=ctr[..., i], order=3
+                    image[..., i], center=ctr[..., i], order=3, spacing=self._spacing
                 )
                 for i in range(image.shape[-1])
             ]
             moments = np.stack(moments_list, axis=-1)
         else:
-            moments = _moments.moments_central(image, ctr, order=3)
+            moments = _moments.moments_central(
+                image, ctr, order=3, spacing=self._spacing
+            )
         return moments
 
     @property
     @only2d
     def moments_weighted_hu(self):
+        if not (np.array(self._spacing) == np.array([1, 1])).all():
+            raise NotImplementedError('`moments_hu` supports spacing = (1, 1) only')
         nu = self.moments_weighted_normalized
         if self._multichannel:
             nchannels = self._intensity_image.shape[-1]
@@ -644,29 +742,22 @@ class RegionProperties:
         if self._multichannel:
             nchannels = self._intensity_image.shape[-1]
             return np.stack(
-                [_moments.moments_normalized(mu[..., i], order=3)
-                 for i in range(nchannels)],
+                [
+                    _moments.moments_normalized(
+                        mu[..., i], order=3, spacing=self._spacing
+                    )
+                    for i in range(nchannels)
+                ],
                 axis=-1,
             )
         else:
-            return _moments.moments_normalized(mu, order=3)
-        return _moments.moments_normalized(self.moments_weighted_central, 3)
+            return _moments.moments_normalized(mu, order=3, spacing=self._spacing)
 
     def __iter__(self):
         props = PROP_VALS
 
         if self._intensity_image is None:
-            unavailable_props = ('image_intensity',
-                                 'intensity_max',
-                                 'intensity_mean',
-                                 'intensity_min',
-                                 'moments_weighted',
-                                 'moments_weighted_central',
-                                 'centroid_weighted',
-                                 'centroid_weighted_local',
-                                 'moments_weighted_hu',
-                                 'moments_weighted_normalized')
-
+            unavailable_props = _require_intensity_image
             props = props.difference(unavailable_props)
 
         return iter(sorted(props))
@@ -685,8 +776,9 @@ class RegionProperties:
         for key in PROP_VALS:
             try:
                 # so that NaNs are equal
-                np.testing.assert_equal(getattr(self, key, None),
-                                        getattr(other, key, None))
+                np.testing.assert_equal(
+                    getattr(self, key, None), getattr(other, key, None)
+                )
             except AssertionError:
                 return False
 
@@ -702,7 +794,7 @@ def _props_to_dict(regions, properties=('label', 'bbox'), separator='-'):
 
     Parameters
     ----------
-    regions : (N,) list
+    regions : (K,) list
         List of RegionProperties objects as returned by :func:`regionprops`.
     properties : tuple or list of str, optional
         Properties that will be included in the resulting dictionary
@@ -807,25 +899,20 @@ def _props_to_dict(regions, properties=('label', 'bbox'), separator='-'):
                 column_buffer[i] = regions[i][prop]
             out[orig_prop] = np.copy(column_buffer)
         else:
-            if isinstance(rp, np.ndarray):
-                shape = rp.shape
-            else:
-                shape = (len(rp),)
-
             # precompute property column names and locations
             modified_props = []
             locs = []
-            for ind in np.ndindex(shape):
-                modified_props.append(
-                    separator.join(map(str, (orig_prop,) + ind))
-                )
+            for ind in np.ndindex(np.shape(rp)):
+                modified_props.append(separator.join(map(str, (orig_prop,) + ind)))
                 locs.append(ind if len(ind) > 1 else ind[0])
 
             # fill temporary column data_array
             n_columns = len(locs)
             column_data = np.empty((n, n_columns), dtype=dtype)
             for k in range(n):
-                rp = regions[k][prop]
+                # we coerce to a numpy array to ensure structures like
+                # tuple-of-arrays expand correctly into columns
+                rp = np.asarray(regions[k][prop])
                 for i, loc in enumerate(locs):
                     column_data[k, i] = rp[loc]
 
@@ -835,10 +922,16 @@ def _props_to_dict(regions, properties=('label', 'bbox'), separator='-'):
     return out
 
 
-def regionprops_table(label_image, intensity_image=None,
-                      properties=('label', 'bbox'),
-                      *,
-                      cache=True, separator='-', extra_properties=None):
+def regionprops_table(
+    label_image,
+    intensity_image=None,
+    properties=('label', 'bbox'),
+    *,
+    cache=True,
+    separator='-',
+    extra_properties=None,
+    spacing=None,
+):
     """Compute image properties and return them as a pandas-compatible table.
 
     The table is a dictionary mapping column names to value arrays. See Notes
@@ -848,13 +941,12 @@ def regionprops_table(label_image, intensity_image=None,
 
     Parameters
     ----------
-    label_image : (N, M[, P]) ndarray
+    label_image : (M, N[, P]) ndarray
         Labeled input image. Labels with value 0 are ignored.
     intensity_image : (M, N[, P][, C]) ndarray, optional
         Intensity (i.e., input) image with same size as labeled image, plus
-        optionally an extra dimension for multichannel data. Currently,
-        this extra channel dimension, if present, must be the last axis.
-        Default is None.
+        optionally an extra dimension for multichannel data. The channel dimension,
+        if present, must be the last axis. Default is None.
 
         .. versionchanged:: 0.18.0
             The ability to provide an extra dimension for channels was added.
@@ -883,10 +975,12 @@ def regionprops_table(label_image, intensity_image=None,
         skimage. The name of the property is derived from the function name,
         the dtype is inferred by calling the function on a small sample.
         If the name of an extra property clashes with the name of an existing
-        property the extra property wil not be visible and a UserWarning is
+        property the extra property will not be visible and a UserWarning is
         issued. A property computation function must take a region mask as its
         first argument. If the property requires an intensity image, it must
         accept the intensity image as the second argument.
+    spacing: tuple of float, shape (ndim,)
+        The pixel spacing along each axis of the image.
 
     Returns
     -------
@@ -971,35 +1065,47 @@ def regionprops_table(label_image, intensity_image=None,
     4      5       112.50        113.0        114.0
 
     """
-    regions = regionprops(label_image, intensity_image=intensity_image,
-                          cache=cache, extra_properties=extra_properties)
+    regions = regionprops(
+        label_image,
+        intensity_image=intensity_image,
+        cache=cache,
+        extra_properties=extra_properties,
+        spacing=spacing,
+    )
     if extra_properties is not None:
-        properties = (
-            list(properties) + [prop.__name__ for prop in extra_properties]
-        )
+        properties = list(properties) + [prop.__name__ for prop in extra_properties]
     if len(regions) == 0:
         ndim = label_image.ndim
         label_image = np.zeros((3,) * ndim, dtype=int)
         label_image[(1,) * ndim] = 1
         if intensity_image is not None:
             intensity_image = np.zeros(
-                    label_image.shape + intensity_image.shape[ndim:],
-                    dtype=intensity_image.dtype
-                    )
-        regions = regionprops(label_image, intensity_image=intensity_image,
-                              cache=cache, extra_properties=extra_properties)
+                label_image.shape + intensity_image.shape[ndim:],
+                dtype=intensity_image.dtype,
+            )
+        regions = regionprops(
+            label_image,
+            intensity_image=intensity_image,
+            cache=cache,
+            extra_properties=extra_properties,
+            spacing=spacing,
+        )
 
-        out_d = _props_to_dict(regions, properties=properties,
-                               separator=separator)
+        out_d = _props_to_dict(regions, properties=properties, separator=separator)
         return {k: v[:0] for k, v in out_d.items()}
 
-    return _props_to_dict(
-        regions, properties=properties, separator=separator
-    )
+    return _props_to_dict(regions, properties=properties, separator=separator)
 
 
-def regionprops(label_image, intensity_image=None, cache=True,
-                coordinates=None, *, extra_properties=None):
+def regionprops(
+    label_image,
+    intensity_image=None,
+    cache=True,
+    *,
+    extra_properties=None,
+    spacing=None,
+    offset=None,
+):
     r"""Measure properties of labeled image regions.
 
     Parameters
@@ -1025,28 +1131,21 @@ def regionprops(label_image, intensity_image=None, cache=True,
         Determine whether to cache calculated properties. The computation is
         much faster for cached properties, whereas the memory consumption
         increases.
-    coordinates : DEPRECATED
-        This argument is deprecated and will be removed in a future version
-        of scikit-image.
-
-        See :ref:`Coordinate conventions <numpy-images-coordinate-conventions>`
-        for more details.
-
-        .. deprecated:: 0.16.0
-            Use "rc" coordinates everywhere. It may be sufficient to call
-            ``numpy.transpose`` on your label image to get the same values as
-            0.15 and earlier. However, for some properties, the transformation
-            will be less trivial. For example, the new orientation is
-            :math:`\frac{\pi}{2}` plus the old orientation.
     extra_properties : Iterable of callables
         Add extra property computation functions that are not included with
         skimage. The name of the property is derived from the function name,
         the dtype is inferred by calling the function on a small sample.
         If the name of an extra property clashes with the name of an existing
-        property the extra property wil not be visible and a UserWarning is
+        property the extra property will not be visible and a UserWarning is
         issued. A property computation function must take a region mask as its
         first argument. If the property requires an intensity image, it must
         accept the intensity image as the second argument.
+    spacing: tuple of float, shape (ndim,)
+        The pixel spacing along each axis of the image.
+    offset : array-like of int, shape `(label_image.ndim,)`, optional
+        Coordinates of the origin ("top-left" corner) of the label image.
+        Normally this is ([0, ]0, 0), but it might be different if one wants
+        to obtain regionprops of subvolumes within a larger volume.
 
     Returns
     -------
@@ -1058,16 +1157,15 @@ def regionprops(label_image, intensity_image=None, cache=True,
     -----
     The following properties can be accessed as attributes or keys:
 
-    **area** : int
-        Number of pixels of the region.
-    **area_bbox** : int
-        Number of pixels of bounding box.
-    **area_convex** : int
-        Number of pixels of convex hull image, which is the smallest convex
+    **area** : float
+        Area of the region i.e. number of pixels of the region scaled by pixel-area.
+    **area_bbox** : float
+        Area of the bounding box i.e. number of pixels of bounding box scaled by pixel-area.
+    **area_convex** : float
+        Area of the convex hull image, which is the smallest convex
         polygon that encloses the region.
-    **area_filled** : int
-        Number of pixels of the region will all the holes filled in. Describes
-        the area of the image_filled.
+    **area_filled** : float
+        Area of the region with all the holes filled in.
     **axis_major_length** : float
         The length of the major axis of the ellipse that has the same
         normalized second central moments as the region.
@@ -1089,7 +1187,9 @@ def regionprops(label_image, intensity_image=None, cache=True,
     **centroid_weighted_local** : array
         Centroid coordinate tuple ``(row, col)``, relative to region bounding
         box, weighted with intensity image.
-    **coords** : (N, 2) ndarray
+    **coords_scaled** : (K, 2) ndarray
+        Coordinate list ``(row, col)``of the region scaled by ``spacing``.
+    **coords** : (K, 2) ndarray
         Coordinate list ``(row, col)`` of the region.
     **eccentricity** : float
         Eccentricity of the ellipse that has the same second-moments as the
@@ -1130,6 +1230,8 @@ def regionprops(label_image, intensity_image=None, cache=True,
         Value with the mean intensity in the region.
     **intensity_min** : float
         Value with the least intensity in the region.
+    **intensity_std** : float
+        Standard deviation of the intensity in the region.
     **label** : int
         The label in the labeled input image.
     **moments** : (3, 3) ndarray
@@ -1178,6 +1280,8 @@ def regionprops(label_image, intensity_image=None, cache=True,
             wnu_ij = wmu_ij / wm_00^[(i+j)/2 + 1]
 
         where ``wm_00`` is the zeroth spatial moment (intensity-weighted area).
+    **num_pixels** : int
+        Number of foreground pixels.
     **orientation** : float
         Angle between the 0th axis (rows) and the major
         axis of the ellipse that has the same second moments as the region,
@@ -1253,30 +1357,25 @@ def regionprops(label_image, intensity_image=None, cache=True,
     if not np.issubdtype(label_image.dtype, np.integer):
         if np.issubdtype(label_image.dtype, bool):
             raise TypeError(
-                    'Non-integer image types are ambiguous: '
-                    'use skimage.measure.label to label the connected'
-                    'components of label_image,'
-                    'or label_image.astype(np.uint8) to interpret'
-                    'the True values as a single label.')
+                'Non-integer image types are ambiguous: '
+                'use skimage.measure.label to label the connected '
+                'components of label_image, '
+                'or label_image.astype(np.uint8) to interpret '
+                'the True values as a single label.'
+            )
         else:
-            raise TypeError(
-                    'Non-integer label_image types are ambiguous')
+            raise TypeError('Non-integer label_image types are ambiguous')
 
-    if coordinates is not None:
-        if coordinates == 'rc':
-            msg = ('The coordinates keyword argument to skimage.measure.'
-                   'regionprops is deprecated. All features are now computed '
-                   'in rc (row-column) coordinates. Please remove '
-                   '`coordinates="rc"` from all calls to regionprops before '
-                   'updating scikit-image.')
-            warn(msg, stacklevel=2, category=FutureWarning)
-        else:
-            msg = ('Values other than "rc" for the "coordinates" argument '
-                   'to skimage.measure.regionprops are no longer supported. '
-                   'You should update your code to use "rc" coordinates and '
-                   'stop using the "coordinates" argument, or use skimage '
-                   'version 0.15.x or earlier.')
-            raise ValueError(msg)
+    if offset is None:
+        offset_arr = np.zeros((label_image.ndim,), dtype=int)
+    else:
+        offset_arr = np.asarray(offset)
+        if offset_arr.ndim != 1 or offset_arr.size != label_image.ndim:
+            raise ValueError(
+                'Offset should be an array-like of integers '
+                'of shape (label_image.ndim,); '
+                f'{offset} was provided.'
+            )
 
     regions = []
 
@@ -1287,8 +1386,16 @@ def regionprops(label_image, intensity_image=None, cache=True,
 
         label = i + 1
 
-        props = RegionProperties(sl, label, label_image, intensity_image,
-                                 cache, extra_properties=extra_properties)
+        props = RegionProperties(
+            sl,
+            label,
+            label_image,
+            intensity_image,
+            cache,
+            spacing=spacing,
+            extra_properties=extra_properties,
+            offset=offset_arr,
+        )
         regions.append(props)
 
     return regions
@@ -1299,8 +1406,9 @@ def _parse_docs():
     import textwrap
 
     doc = regionprops.__doc__ or ''
-    matches = re.finditer(r'\*\*(\w+)\*\* \:.*?\n(.*?)(?=\n    [\*\S]+)',
-                          doc, flags=re.DOTALL)
+    matches = re.finditer(
+        r'\*\*(\w+)\*\* \:.*?\n(.*?)(?=\n    [\*\S]+)', doc, flags=re.DOTALL
+    )
     prop_doc = {m.group(1): textwrap.dedent(m.group(2)) for m in matches}
 
     return prop_doc
@@ -1309,8 +1417,7 @@ def _parse_docs():
 def _install_properties_docs():
     prop_doc = _parse_docs()
 
-    for p in [member for member in dir(RegionProperties)
-              if not member.startswith('_')]:
+    for p in [member for member in dir(RegionProperties) if not member.startswith('_')]:
         getattr(RegionProperties, p).__doc__ = prop_doc[p]
 
 
