@@ -4,8 +4,7 @@ import numpy as np
 __all__ = ['contingency_table']
 
 
-def contingency_table(im_true, im_test, *, ignore_labels=None,
-                      normalize=False):
+def contingency_table(im_true, im_test, *, ignore_labels=None, normalize=False):
     """
     Return the contingency table for all regions in matched segmentations.
 
