@@ -1,6 +1,7 @@
-
 import numpy as np
 from scipy import ndimage as ndi
+
+from .._shared.utils import _supported_float_type
 from ..morphology import dilation, erosion, square
 from ..util import img_as_float, view_as_windows
 from ..color import gray2rgb
@@ -24,17 +25,16 @@ def _find_boundaries_subpixel(label_img):
     ndim = label_img.ndim
     max_label = np.iinfo(label_img.dtype).max
 
-    label_img_expanded = np.zeros([(2 * s - 1) for s in label_img.shape],
-                                  label_img.dtype)
-    pixels = (slice(None, None, 2), ) * ndim
+    label_img_expanded = np.zeros(
+        [(2 * s - 1) for s in label_img.shape], label_img.dtype
+    )
+    pixels = (slice(None, None, 2),) * ndim
     label_img_expanded[pixels] = label_img
 
     edges = np.ones(label_img_expanded.shape, dtype=bool)
     edges[pixels] = False
     label_img_expanded[edges] = max_label
-    windows = view_as_windows(np.pad(label_img_expanded, 1,
-                                     mode='constant', constant_values=0),
-                              (3,) * ndim)
+    windows = view_as_windows(np.pad(label_img_expanded, 1, mode='edge'), (3,) * ndim)
 
     boundaries = np.zeros_like(edges)
     for index in np.ndindex(label_img_expanded.shape):
@@ -147,7 +147,8 @@ def find_boundaries(label_img, connectivity=1, mode='thick', background=0):
     ...                        [False, False, False, False, False],
     ...                        [False, False,  True,  True,  True],
     ...                        [False, False,  True,  True,  True],
-    ...                        [False, False,  True,  True,  True]], dtype=np.bool)
+    ...                        [False, False,  True,  True,  True]],
+    ...                       dtype=bool)
     >>> find_boundaries(bool_image)
     array([[False, False, False, False, False],
            [False, False,  True,  True,  True],
@@ -158,30 +159,37 @@ def find_boundaries(label_img, connectivity=1, mode='thick', background=0):
     if label_img.dtype == 'bool':
         label_img = label_img.astype(np.uint8)
     ndim = label_img.ndim
-    selem = ndi.generate_binary_structure(ndim, connectivity)
+    footprint = ndi.generate_binary_structure(ndim, connectivity)
     if mode != 'subpixel':
-        boundaries = dilation(label_img, selem) != erosion(label_img, selem)
+        boundaries = dilation(label_img, footprint) != erosion(label_img, footprint)
         if mode == 'inner':
-            foreground_image = (label_img != background)
+            foreground_image = label_img != background
             boundaries &= foreground_image
         elif mode == 'outer':
             max_label = np.iinfo(label_img.dtype).max
-            background_image = (label_img == background)
-            selem = ndi.generate_binary_structure(ndim, ndim)
+            background_image = label_img == background
+            footprint = ndi.generate_binary_structure(ndim, ndim)
             inverted_background = np.array(label_img, copy=True)
             inverted_background[background_image] = max_label
-            adjacent_objects = ((dilation(label_img, selem) !=
-                                 erosion(inverted_background, selem)) &
-                                ~background_image)
-            boundaries &= (background_image | adjacent_objects)
+            adjacent_objects = (
+                dilation(label_img, footprint)
+                != erosion(inverted_background, footprint)
+            ) & ~background_image
+            boundaries &= background_image | adjacent_objects
         return boundaries
     else:
         boundaries = _find_boundaries_subpixel(label_img)
         return boundaries
 
 
-def mark_boundaries(image, label_img, color=(1, 1, 0),
-                    outline_color=None, mode='outer', background_label=0):
+def mark_boundaries(
+    image,
+    label_img,
+    color=(1, 1, 0),
+    outline_color=None,
+    mode='outer',
+    background_label=0,
+):
     """Return image with boundaries between labeled regions highlighted.
 
     Parameters
@@ -211,7 +219,9 @@ def mark_boundaries(image, label_img, color=(1, 1, 0),
     --------
     find_boundaries
     """
+    float_dtype = _supported_float_type(image.dtype)
     marked = img_as_float(image, force_copy=True)
+    marked = marked.astype(float_dtype, copy=False)
     if marked.ndim == 2:
         marked = gray2rgb(marked)
     if mode == 'subpixel':
@@ -219,10 +229,10 @@ def mark_boundaries(image, label_img, color=(1, 1, 0),
         # each original line - except for the last axis which holds
         # the RGB information. ``ndi.zoom`` then performs the (cubic)
         # interpolation, filling in the values of the interposed pixels
-        marked = ndi.zoom(marked, [2 - 1/s for s in marked.shape[:-1]] + [1],
-                          mode='reflect')
-    boundaries = find_boundaries(label_img, mode=mode,
-                                 background=background_label)
+        marked = ndi.zoom(
+            marked, [2 - 1 / s for s in marked.shape[:-1]] + [1], mode='mirror'
+        )
+    boundaries = find_boundaries(label_img, mode=mode, background=background_label)
     if outline_color is not None:
         outlines = dilation(boundaries, square(3))
         marked[outlines] = outline_color
